﻿/*  BBWin Gui version 2.0
    Copyright 2012 by Craig Kergald, and Shannon Anderson
    bbwingui@kergald.net
    Any changes to this software should include this header
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


using System;
using System.Diagnostics;
using System.Threading;
using System.ServiceProcess;
using System.Management;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Web;
using System.Net;
using System.Text.RegularExpressions;
using System.IO;
using System.Xml;
using System.Windows.Forms;

namespace WindowsFormsApplication2
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
        public class Globals
        {
            // C# does not have global variables, 
            // This is a work around, access them using this example
            // Globals.Saved = value;
            //
            public static bool Saved = false;
            public static bool changed = false;
            public static string xml_doc = "";
            public static bool ready = false;
            public static string defaults_url = "http://www.squidworks.net/BBWin.cfg.template";
            public static string services_master_list_url = "http://www.squidworks.net/Master_List_Of_Services.txt";
            public static string version = "2.2";
            public static string[] floppers = { "Shell Hardware Detection", "Windows Modules Installer", "WinHTTP Web Proxy Auto-Discovery Service", "Software Protection", "Block Level Backup Engine Service", "Office Source Engine", "File Server Storage Reports Manager" };

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // if close X has been pressed
            // ask to save changes if globals.changed is true
            if (Globals.changed == true)
            {
                e.Cancel = true;
                DialogResult result = MessageBox.Show("Save changes ?", "Save Changes ?", MessageBoxButtons.YesNo);
                if (result == DialogResult.No) { Globals.changed = false; Application.Exit(); }
                if (result == DialogResult.Yes)
                {
                    string fname = @"c:\";
                    if (File.Exists(@"c:\program files\bbwin\etc\bbwin.cfg")) { fname = @"c:\program files\bbwin\etc\"; }
                    if (File.Exists(@"c:\program files (x86)\bbwin\etc\bbwin.cfg")) { fname = @"c:\program files (x86)\bbwin\etc\"; }
                    SaveFileDialog savedialog = new SaveFileDialog();
                    savedialog.Title = "Save As";
                    savedialog.DefaultExt = "cfg";
                    savedialog.InitialDirectory = fname;
                    savedialog.Filter = "cfg files (*.cfg)|*.cfg|bak files (*.bak)|*.bak|All files (*.*)|*.*";
                    string filename = "";
                    if (savedialog.ShowDialog() == DialogResult.OK)
                    {
                        filename = savedialog.FileName;
                    }
                    if (filename != "")
                    {
                        string backup_filename = backup_config(filename);
                        XmlDocument doc = new XmlDocument();
                        doc.LoadXml(Globals.xml_doc);
                        doc.Save(filename);
                        Globals.changed = false;
                        if (backup_filename != "")
                        { MessageBox.Show("Configuration saved to: " + filename + "\n" + backup_filename); }
                        else { MessageBox.Show("Configuration saved to: " + filename); }
                    }
                }
            }
        } 
        private void Form1_Load(object sender, EventArgs e)
        {
            Set_defaults();
            this.Text = "BBWin Gui Version " + Globals.version;
            build_disk_listview("","","","create");
            bool bb_win_installed = Check_BBWin_Installed();
            // if bb_win_installed is false ask to install bbwin
            if (bb_win_installed == false)
            {
                DialogResult result = MessageBox.Show("BBwin not found, would you like to download it ?", "BBWin missing", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes) { System.Diagnostics.Process.Start("http://sourceforge.net/projects/bbwin/"); }
            }
            Startup_Selection sselection = new Startup_Selection();
            sselection.Show();
            int schoice = sselection.startup_choice();
            sselection.Hide();
            if (schoice == 0)
            {
                OpenFileDialog fdlg = new OpenFileDialog();
                string fname = @"c:\";
                if (File.Exists(@"c:\program files\bbwin\etc\bbwin.cfg")) { fname = @"c:\program files\bbwin\etc\"; }
                if (File.Exists(@"c:\program files (x86)\bbwin\etc\bbwin.cfg")) { fname = @"c:\program files (x86)\bbwin\etc\"; }
                fdlg.Title = "Open configuration file";
                fdlg.InitialDirectory = fname;
                fdlg.Filter = "All files (*.*)|*.*|All files (*.*)|*.*";
                fdlg.FilterIndex = 2;
                fdlg.RestoreDirectory = true;
                if (fdlg.ShowDialog() == DialogResult.OK)
                {
                    string filename = fdlg.FileName;
                    Read_Config_File(filename);
                }
            }
            if (schoice == 2)
            {
                string filename = Globals.defaults_url;
                Read_Config_File(filename);
            }   
        }

        private void Read_Config_File(string filename)
        {
            //
            // READ XML CONFIGURATION FILE
            // 
            if (filename.Contains("http://") == false)
            {
                fix_bad_xml(filename);
            }
            try
            {   
                XmlDocument doc = new XmlDocument();
                doc.Load(filename);
                //check if were loading defaults from web or a file from disk
                if (filename.Contains("http://"))
                {
                    using (var client = new WebClient())
                    {
                        Globals.xml_doc = client.DownloadString(filename);
                    }
                }
                else
                {
                    using (System.IO.StreamReader sr = new System.IO.StreamReader(filename))
                    {
                        Globals.xml_doc = sr.ReadToEnd();
                    }
                }
                XmlNodeList settings = doc.SelectNodes("//bbwin//setting");
                foreach (XmlNode setting in settings)
                {
                    // Read settings from bbwin section
                    // pass this to update_text_box routine
                    Update_Text_Box(setting.Attributes["name"].Value, setting.Attributes["value"].Value,"bbwin//setting");
                }
                settings = doc.SelectNodes("//bbwin//load");
                foreach (XmlNode setting in settings)
                {
                    Update_Text_Box(setting.Attributes["name"].Value, setting.Attributes["value"].Value, "bbwin//load");
                }
                settings = doc.SelectNodes("//cpu//setting");
                foreach (XmlNode setting in settings)
                {
                    if (setting.Attributes["name"].Value == "alwaysgreen")
                    {
                        Update_Text_Box(setting.Attributes["name"].Value, setting.Attributes["value"].Value, "cpu//setting");
                    }
                    if (setting.Attributes["name"].Value == "default")
                    {
                        Update_Text_Box("warnlevel", setting.Attributes["warnlevel"].Value, "cpu//setting");
                        Update_Text_Box("paniclevel", setting.Attributes["paniclevel"].Value, "cpu//setting");
                    }
                }
                settings = doc.SelectNodes("//disk//setting");
                foreach (XmlNode setting in settings)
                {
                    if (setting.Attributes["name"].Value == "alwaysgreen")
                    {}
                    else if (setting.Attributes["name"].Value == "remote"){}
                    else if (setting.Attributes["name"].Value == "cdrom") {}
                    else
                    {
                        build_disk_listview(setting.Attributes["name"].Value, setting.Attributes["warnlevel"].Value, setting.Attributes["paniclevel"].Value, "add");
                    }
                }
                settings = doc.SelectNodes("//memory//setting");
                foreach (XmlNode setting in settings)
                {
                    if (setting.Attributes["name"].Value == "alwaysgreen")
                    {
                        Update_Text_Box(setting.Attributes["name"].Value, setting.Attributes["value"].Value, "memory//setting");
                    }
                    if (setting.Attributes["name"].Value == "physical")
                    {
                        Update_Text_Box("warnlevel", setting.Attributes["warnlevel"].Value, "memory//setting//physical");
                        Update_Text_Box("paniclevel", setting.Attributes["paniclevel"].Value, "memory//setting//physical");
                    }
                    if (setting.Attributes["name"].Value == "page")
                    {
                        Update_Text_Box("warnlevel", setting.Attributes["warnlevel"].Value, "memory//setting//page");
                        Update_Text_Box("paniclevel", setting.Attributes["paniclevel"].Value, "memory//setting//page");
                    }
                    if (setting.Attributes["name"].Value == "virtual")
                    {
                        Update_Text_Box("warnlevel", setting.Attributes["warnlevel"].Value, "memory//setting//virtual");
                        Update_Text_Box("paniclevel", setting.Attributes["paniclevel"].Value, "memory//setting//virtual");
                    }
                }
                settings = doc.SelectNodes("//svcs//setting");
                foreach (XmlNode setting in settings)
                {
                    Update_Text_Box(setting.Attributes["name"].Value, "", "svcs//setting");
                }
                settings = doc.SelectNodes("//procs//setting");
                foreach (XmlNode setting in settings)
                {
                    string temp_value = setting.Attributes["name"].Value + "|" + setting.Attributes["rule"].Value;
                    try{
                    if (setting.Attributes["comment"].Value != "") { temp_value = temp_value + "|" + setting.Attributes["comment"].Value; }
                    }
                    catch { temp_value = temp_value + "|no description available"; }
                    Update_Text_Box(temp_value, "", "procs//setting");
                }
                Globals.ready = true;
                if (filename.Contains("http://"))
                {
                    hostname.Text = System.Net.Dns.GetHostName().ToString();
                    if (System.Net.NetworkInformation.IPGlobalProperties.GetIPGlobalProperties().DomainName.ToString() != "")
                    { hostname.Text = hostname.Text + "." + System.Net.NetworkInformation.IPGlobalProperties.GetIPGlobalProperties().DomainName.ToString(); }
                }
                Globals.changed = false;
                RTB.Clear();
                RTB.Text = Globals.xml_doc;
            }
            catch
            {
                MessageBox.Show("An error occured reading file-" + filename +"\nThis usually happens due to misformed XML\nFor example the & character should be encoded as &amp;");
            }
        }
        private void fix_bad_xml(string filename)
        {
            // replace any & symbols with &amp;
            // as this is proper xml coding and
            // we have several deployed xml files
            // with the & in them, encoded improperly
            // bbwin handles this without problems,
            // but C# will throw an exception trying to
            // read a xml file with an & in it
            StreamReader streamReader = new StreamReader(filename);
            string text = streamReader.ReadToEnd();
            streamReader.Close();
            Regex badAmpersand = new Regex("&(?![a-zA-Z]{2,6};|#[0-9]{2,4};)");
            const string goodAmpersand = "&amp;";
            text = badAmpersand.Replace(text, goodAmpersand);
            Regex rgx = new Regex(">(?<prefix>.*)&(?<sufix>.*)<"); 
            text = rgx.Replace(text, ">${prefix}&amp;${sufix}<"); 
            StreamWriter streamWriter = new StreamWriter(filename);
            streamWriter.WriteLine(text);
            streamWriter.Close();
            
        }
        private Boolean Check_BBWin_Installed()
        {
            // checks if bbwin service is listed as
            // a windows service, returns true or false
            ServiceController[] services = ServiceController.GetServices();
            bool returnvalue = false;
            foreach (ServiceController service in services)
            {
                if (service.DisplayName == "BBWin") { returnvalue = true; }
                if (service.DisplayName == "Big Brother Xymon Client") { returnvalue = true; }
                if (service.DisplayName == "Big Brother Hobbit Client") { returnvalue = true; }
            }
            return returnvalue;
        }

        private void Update_Text_Box(string box_name, string text_value, string section)
        {
            // Information passed into this subroutine will be used
            // to populate text fields in the gui
            //
            // the text box name and value are passed in
            if (section == "bbwin//setting")
            {
                if (box_name == "bbdisplay") { bbdisplay.Text = text_value; }
                if (box_name == "hostname") { hostname.Text = text_value; }
                if ((box_name == "mode") & (text_value == "local")) { Mode_Combo_Box.SelectedIndex = 0; }
                if ((box_name == "mode") & (text_value == "central")) { Mode_Combo_Box.SelectedIndex = 1; }
                if (box_name == "configclass") { configclass.Text = text_value; }
                if ((box_name == "autoreload") & (text_value == "true")) { autoreload_cb.CheckState = CheckState.Checked; }
                if ((box_name == "autoreload") & (text_value == "false")) { autoreload_cb.CheckState = CheckState.Unchecked; }
                if (box_name == "timer") { timer.Text = text_value; }
                if (box_name == "loglevel") { loglevel.Text = text_value; }
                if (box_name == "logpath") { logpath.Text = text_value; }
                if ((box_name == "logreportfailure") & (text_value == "true")) { logreportfailure.CheckState = CheckState.Checked; }
                if ((box_name == "logreportfailure") & (text_value == "false")) { logreportfailure.CheckState = CheckState.Unchecked; }
            }
            if (section == "bbwin//load")
            {
                if (box_name == "cpu") { cpu_dll.Text = text_value; }
                if (box_name == "disk") { disk_dll.Text = text_value; }
                if (box_name == "externals") { externals_dll.Text = text_value; }
                if (box_name == "filesystem") { filesystem_dll.Text = text_value; }
                if (box_name == "memory") { memory_dll.Text = text_value; }
                if (box_name == "msgs") { msgs_dll.Text = text_value; }
                if (box_name == "procs") { procs_dll.Text = text_value; }
                if (box_name == "stats") { stats_dll.Text = text_value; }
                if (box_name == "svcs") { svcs_dll.Text = text_value; }
                if (box_name == "uptime") { uptime_dll.Text = text_value; }
                if (box_name == "who") { who_dll.Text = text_value; }
            }
            if (section == "cpu//setting")
            {
                if (box_name == "alwaysgreen")
                {
                    if (text_value == "true")
                    { cpu_always_Green_CB.CheckState = CheckState.Checked; }
                    else { cpu_always_Green_CB.CheckState = CheckState.Unchecked; }
                }
                if (box_name == "warnlevel")
                {
                    cpu_warn.Value = int.Parse(text_value);
                }
                if (box_name == "paniclevel")
                {
                    cpu_panic.Value = int.Parse(text_value);
                }
            }
            if (section == "memory//setting")
            {
                if (box_name == "alwaysgreen")
                {
                    if (text_value == "true")
                    { 
                        Memory_always_green_CB.CheckState = CheckState.Checked;
                        Memory_always_green_CB_CheckedChanged(null,null);
                    }
                    else { 
                        Memory_always_green_CB.CheckState = CheckState.Unchecked;
                        Memory_always_green_CB_CheckedChanged(null,null);
                    }
                }
            }
            if (section == "memory//setting//physical")
            {
                if (box_name == "warnlevel")
                {
                    Memory_physical_warn.Value = int.Parse(text_value);
                }
                if (box_name == "paniclevel")
                {
                    memory_physical_panic.Value = int.Parse(text_value);
                }
            }
            if (section == "memory//setting//page")
            {
                if (box_name == "warnlevel")
                {
                    Mem_page_warning_level.Value = int.Parse(text_value);
                }
                if (box_name == "paniclevel")
                {
                    Mem_Page_panic_level.Value = int.Parse(text_value);
                }
            }
            if (section == "memory//setting//virtual")
            {
                if (box_name == "warnlevel")
                {
                    Memory_Virtual_warn.Value = int.Parse(text_value);
                }
                if (box_name == "paniclevel")
                {
                    Memory_Virtual_panic.Value = int.Parse(text_value);
                }
            }
            if (section == "svcs//setting")
            {
                if ((box_name != "alwaysgreen") && (box_name != "autoreset") && (box_name != "alarmcolor"))
                {
                LB_Services_to_monitor.Items.Add(box_name);
                }
            }
            if (section == "procs//setting")
            {
                string[] temp_value = box_name.Split('|');
                add_proc(temp_value[0], temp_value[1], temp_value[2]);
            }
        }

        private void Set_defaults()
        {
            // sets all gui object to default state.
            bbdisplay.Text = "";
            hostname.Text = "";
            Mode_Combo_Box.SelectedIndex = 0;
            configclass.Text = "";
            configclass.Enabled = false;
            autoreload_cb.CheckState = CheckState.Checked;
            timer.Text = "";
            loglevel.Text = "";
            logpath.Text = "";
            disk_dll.Text = "";
            cpu_dll.Text = "";
            filesystem_dll.Text = "";
            externals_dll.Text = "";
            memory_dll.Text = "";
            msgs_dll.Text = "";
            stats_dll.Text = "";
            procs_dll.Text = "";
            svcs_dll.Text = "";
            uptime_dll.Text = "";
            who_dll.Text = "";
            Disk_percent_warn_rb.Checked = true;
            disk_panic_textbox.Visible = false;
            disk_warn_textbox.Visible = false;
            dlls_GB.Enabled = false;
            RTB.Visible = true;
            logreportfailure.CheckState = CheckState.Unchecked;
            Globals.Saved = false;
            Globals.changed = false;
            gb_Disk_settings.Enabled = false;
            disk_remove.Enabled = false;
            label35.Text = "BBWin Gui Version " + Globals.version;
        }


        private void general_settings_advanced_CheckedChanged(object sender, EventArgs e)
        {
            if (general_settings_advanced.CheckState == CheckState.Checked)
            { dlls_GB.Enabled = true; configclass.Enabled = true; }
            else
            { dlls_GB.Enabled = false; configclass.Enabled = false; }
        }


        private void Anything_Changed(string text_box_name, string text_value, string section, string value_name)
        {
            //
            // this section takes in the name of the text box being changed, it's value, 
            // the section of xml needing to be modified, and a value_name used to create
            // an xml attribute if needed

            if (Globals.ready == true) // checks to ensure basic startup checks and pre-population complete
            {
                Globals.changed = true;
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(Globals.xml_doc);
                XmlNodeList settings = doc.SelectNodes(section);
                bool attribute_exists = false;
                foreach (XmlNode setting in settings)
                {
                    if (setting.Attributes["name"].Value == text_box_name)
                    {
                        try
                        {
                            setting.Attributes[value_name].Value = text_value;
                            StringWriter sw = new StringWriter();
                            XmlTextWriter tx = new XmlTextWriter(sw);
                            tx.Formatting = Formatting.Indented;
                            doc.WriteTo(tx);
                            Globals.xml_doc = sw.ToString();
                            tx.Flush();
                            attribute_exists = true;
                        }
                        catch { attribute_exists = false; }
                    }
                }
                if (attribute_exists == false)
                {
                    XmlNode node = doc.SelectNodes(section.Replace("//setting",""))[0];
                    XmlElement new_element = doc.CreateElement("setting");
                    new_element.SetAttribute("name", text_box_name);
                    new_element.SetAttribute(value_name, text_value);
                    node.AppendChild(new_element);
                    StringWriter sw = new StringWriter();
                    XmlTextWriter tx = new XmlTextWriter(sw);
                    tx.Formatting = Formatting.Indented;
                    doc.WriteTo(tx);
                    Globals.xml_doc = sw.ToString();
                    tx.Flush();
                }
                RTB.Clear();
                RTB.Text = Globals.xml_doc; // refresh xml box on xml file tab
            }
        }

        //
        // these events are for when text fields are changed 
        // we call anything changed routine above
        // to make changed to xml

        private void bbdisplay_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("bbdisplay", bbdisplay.Text, "configuration//bbwin//setting", "value");
        }
        private void hostname_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("hostname", hostname.Text, "configuration//bbwin//setting", "value");
        }
       
        private void logpath_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("logpath", logpath.Text, "configuration//bbwin//setting", "value");
        }

        private void loglevel_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("loglevel", loglevel.Text, "configuration//bbwin//setting", "value");
        }

        private void Mode_Combo_Box_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("mode", Mode_Combo_Box.Text, "configuration//bbwin//setting", "value");
        }

        private void configclass_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("configclass", configclass.Text, "configuration//bbwin//setting", "value");
        }

        private void timer_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("timer", timer.Text, "configuration//bbwin//setting", "value");
        }

        private void autoreload_cb_CheckedChanged(object sender, EventArgs e)
        {
            if (autoreload_cb.CheckState == CheckState.Checked)
            { Anything_Changed("autoreload", "true", "configuration//bbwin//setting", "value"); }
            else { Anything_Changed("autoreload", "false", "configuration//bbwin//setting", "value"); }
        }

        private void cpu_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("cpu", cpu_dll.Text, "configuration//bbwin//load", "value");
        }

        private void disk_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("disk", disk_dll.Text, "configuration//bbwin//load", "value");
        }

        private void externals_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("externals", externals_dll.Text, "configuration//bbwin//load", "value");
        }

        private void filesystem_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("filesystem", filesystem_dll.Text, "configuration//bbwin//load", "value");
        }

        private void memory_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("memory", memory_dll.Text, "configuration//bbwin//load", "value");
        }

        private void msgs_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("msgs", msgs_dll.Text, "configuration//bbwin//load", "value");
        }

        private void procs_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("procs", procs_dll.Text, "configuration//bbwin//load", "value");
        }

        private void stats_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("stats", stats_dll.Text, "configuration//bbwin//load", "value");
        }

        private void svcs_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("svcs", svcs_dll.Text, "configuration//bbwin//load", "value");
        }

        private void uptime_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("uptime", uptime_dll.Text, "configuration//bbwin//load", "value");
        }

        private void who_dll_TextChanged(object sender, EventArgs e)
        {
            Anything_Changed("who", who_dll.Text, "configuration//bbwin//load", "value");
        }

        private void logreportfailure_CheckStateChanged(object sender, EventArgs e)
        {
            if (logreportfailure.CheckState == CheckState.Checked)
            { Anything_Changed("logreportfailure", "true", "configuration//bbwin//setting", "value"); }
            else { Anything_Changed("logreportfailure", "false", "configuration//bbwin//setting", "value"); }
        }

        private void cpu_warn_Scroll(object sender, EventArgs e)
        {
            Anything_Changed("default", cpu_warn.Value.ToString(), "configuration//cpu//setting", "warnlevel");
            if (cpu_panic.Value <= cpu_warn.Value) { cpu_panic.Value = cpu_warn.Value; }
            toolTip1.SetToolTip(cpu_warn, cpu_warn.Value.ToString());
        }

        private void cpu_panic_Scroll(object sender, EventArgs e)
        {
            Anything_Changed("default", cpu_panic.Value.ToString(), "configuration//cpu//setting", "paniclevel");
            if (cpu_warn.Value >= cpu_panic.Value) { cpu_warn.Value = cpu_panic.Value; }
            toolTip1.SetToolTip(cpu_panic, cpu_panic.Value.ToString());
        }

        private void cpu_always_Green_CB_CheckedChanged(object sender, EventArgs e)
        {
            if (cpu_always_Green_CB.CheckState == CheckState.Checked)
            {
                cpu_panic.Enabled = false;
                cpu_warn.Enabled = false;
                Anything_Changed("alwaysgreen", "true", "configuration//cpu//setting", "value");
                Anything_Changed("alwaysgreen", "false", "configuration//cpu//setting", "value");
            }
            else
            {
                cpu_panic.Enabled = true;
                cpu_warn.Enabled = true;
                Anything_Changed("alwaysgreen", "false", "configuration//cpu//setting", "value");
                Anything_Changed("alwaysgreen", "true", "configuration//cpu//setting", "value");
            }
        }

        private void memory_physical_panic_Scroll(object sender, EventArgs e)
        {
            toolTip1.SetToolTip(memory_physical_panic, memory_physical_panic.Value.ToString());
            if (memory_physical_panic.Value <= Memory_physical_warn.Value) { Memory_physical_warn.Value = memory_physical_panic.Value; }
            Anything_Changed("physical", memory_physical_panic.Value.ToString(), "configuration//memory//setting", "paniclevel");
            Anything_Changed("physical", Memory_physical_warn.Value.ToString(), "configuration//memory/setting", "warnlevel");
        }

        private void Memory_physical_warn_Scroll(object sender, EventArgs e)
        {
            toolTip1.SetToolTip(Memory_physical_warn, Memory_physical_warn.Value.ToString());
            if (memory_physical_panic.Value <= Memory_physical_warn.Value) { memory_physical_panic.Value = Memory_physical_warn.Value; }
            Anything_Changed("physical", Memory_physical_warn.Value.ToString(), "configuration//memory/setting", "warnlevel");
            Anything_Changed("physical", memory_physical_panic.Value.ToString(), "configuration//memory//setting", "paniclevel");
        }

        private void Mem_page_warning_level_Scroll(object sender, EventArgs e)
        {
            toolTip1.SetToolTip(Mem_page_warning_level, Mem_page_warning_level.Value.ToString());
            if (Mem_Page_panic_level.Value <= Mem_page_warning_level.Value) { Mem_Page_panic_level.Value = Mem_page_warning_level.Value; }
            Anything_Changed("page", Mem_page_warning_level.Value.ToString(), "configuration//memory//setting", "warnlevel");
            Anything_Changed("page", Mem_Page_panic_level.Value.ToString(), "configuration//memory//setting", "paniclevel");
        }

        private void Mem_Page_panic_level_Scroll(object sender, EventArgs e)
        {
            toolTip1.SetToolTip(Mem_Page_panic_level, Mem_Page_panic_level.Value.ToString());
            if (Mem_page_warning_level.Value >= Mem_Page_panic_level.Value) { Mem_page_warning_level.Value = Mem_Page_panic_level.Value; }
            Anything_Changed("page", Mem_page_warning_level.Value.ToString(), "configuration//memory//setting", "warnlevel");
            Anything_Changed("page", Mem_Page_panic_level.Value.ToString(), "configuration//memory//setting", "paniclevel");  
        }

        private void Memory_Virtual_warn_Scroll(object sender, EventArgs e)
        {
            toolTip1.SetToolTip(Memory_Virtual_warn, Memory_Virtual_warn.Value.ToString());
            if (Memory_Virtual_panic.Value <= Memory_Virtual_warn.Value) { Memory_Virtual_panic.Value = Memory_Virtual_warn.Value; }
            Anything_Changed("virtual", Memory_Virtual_warn.Value.ToString(), "configuration//memory/setting", "warnlevel");
            Anything_Changed("virtual", Memory_Virtual_panic.Value.ToString(), "configuration//memory//setting", "paniclevel");
        }

        private void Memory_Virtual_panic_Scroll(object sender, EventArgs e)
        {
            toolTip1.SetToolTip(Memory_Virtual_panic, Memory_Virtual_panic.Value.ToString());
            if (Memory_Virtual_warn.Value >= Memory_Virtual_panic.Value) { Memory_Virtual_warn.Value = Memory_Virtual_panic.Value; }
            Anything_Changed("virtual", Memory_Virtual_warn.Value.ToString(), "configuration//memory/setting", "warnlevel");
            Anything_Changed("virtual", Memory_Virtual_panic.Value.ToString(), "configuration//memory//setting", "paniclevel");

        }

        private void Memory_always_green_CB_CheckedChanged(object sender, EventArgs e)
        {
            if (Memory_always_green_CB.CheckState == CheckState.Checked)
            {
                groupBox5.Enabled = false;
                groupBox6.Enabled = false;
                groupBox7.Enabled = false;
                Anything_Changed("alwaysgreen", "true", "configuration//memory//setting", "value");
            }
            else
            {
                groupBox5.Enabled = true;
                groupBox6.Enabled = true;
                groupBox7.Enabled = true;
                Anything_Changed("alwaysgreen", "false", "configuration//memory//setting", "value");
            }
        }

        private void Disk_percent_warn_rb_CheckedChanged(object sender, EventArgs e)
        {
            if (Disk_percent_warn_rb.Checked == true)
            {
                disk_warn_textbox.Visible = false;
                disk_panic_textbox.Visible = false;
                disk_panic_trackbar.Visible = true;
                disk_warn_trackbar.Visible = true;
                Disk_MB_warn_RB.Checked = false;
            }
        }

        private void Disk_MB_warn_RB_CheckedChanged(object sender, EventArgs e)
        {
            if (Disk_MB_warn_RB.Checked == true)
            {
                disk_warn_textbox.Visible = true;
                disk_panic_textbox.Visible = true;
                disk_panic_trackbar.Visible = false;
                disk_warn_trackbar.Visible = false;
                Disk_percent_warn_rb.Checked = false;
            }
        }

        private void disk_warn_trackbar_Scroll(object sender, EventArgs e)
        {
            toolTip1.SetToolTip(disk_warn_trackbar, disk_warn_trackbar.Value.ToString());
            if (disk_panic_trackbar.Value <= disk_warn_trackbar.Value) { disk_panic_trackbar.Value = disk_warn_trackbar.Value; }
            build_disk_listview(disk_LV.Items[disk_LV.SelectedIndices[0]].Text, disk_warn_trackbar.Value.ToString() + "%", disk_panic_trackbar.Value.ToString() + "%", "change");
        }

        private void disk_panic_trackbar_Scroll(object sender, EventArgs e)
        {
            toolTip1.SetToolTip(disk_panic_trackbar, disk_panic_trackbar.Value.ToString());
            if (disk_warn_trackbar.Value >= disk_panic_trackbar.Value) { disk_warn_trackbar.Value = disk_panic_trackbar.Value; }
            build_disk_listview(disk_LV.Items[disk_LV.SelectedIndices[0]].Text, disk_warn_trackbar.Value.ToString() + "%", disk_panic_trackbar.Value.ToString() + "%", "change");
        }
        private void build_disk_listview(string name,string warn,string panic,string task)
        {
            if (task == "create")
            {
                disk_LV.View = View.Details;
                disk_LV.FullRowSelect = true;
                disk_LV.GridLines = true;
                disk_LV.MultiSelect = false;
                disk_LV.Columns.Add("Name");
                disk_LV.Columns.Add("Warning Level");
                disk_LV.Columns.Add("Panic Level");
                disk_LV.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.HeaderSize);
                disk_LV.AutoResizeColumn(1, ColumnHeaderAutoResizeStyle.HeaderSize);
                disk_LV.AutoResizeColumn(2, ColumnHeaderAutoResizeStyle.HeaderSize);
            }
            if (task == "add")
            {
                ListViewItem LVI = new ListViewItem();
                LVI.Text = name;
                LVI.SubItems.Add(warn);
                LVI.SubItems.Add(panic);
                disk_LV.Items.Add(LVI);
            }
            if (task == "change")
            {
                disk_LV.Items[disk_LV.SelectedIndices[0]].Text = name;
                disk_LV.Items[disk_LV.SelectedIndices[0]].SubItems[1].Text = warn;
                disk_LV.Items[disk_LV.SelectedIndices[0]].SubItems[2].Text = panic;
                Anything_Changed(disk_LV.Items[disk_LV.SelectedIndices[0]].Text, warn, "configuration//disk//setting", "warnlevel");
                Anything_Changed(disk_LV.Items[disk_LV.SelectedIndices[0]].Text, panic, "configuration//disk//setting", "paniclevel");
            }
        }

        private void disk_LV_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (disk_LV.SelectedItems.Count == 0)
            {
                gb_Disk_settings.Enabled = false;
                disk_remove.Enabled = false;
                return;
            }
            string name = disk_LV.SelectedItems[0].Text.ToString();
            string warn = disk_LV.SelectedItems[0].SubItems[1].Text.ToString();
            string panic = disk_LV.SelectedItems[0].SubItems[2].Text.ToString();
            string mode = "mb";
            if (warn.Contains("%") == true) { mode = "%"; }
            if (panic.Contains("%") == true) {mode = "%";}
            if (mode == "mb")
            {
                disk_panic_textbox.Visible = true;
                disk_panic_trackbar.Visible = false;
                disk_warn_textbox.Visible = true;
                disk_warn_trackbar.Visible = false;
                Disk_percent_warn_rb.Checked = false;
                Disk_MB_warn_RB.Checked = true;
                warn = warn.ToLower();
                warn = warn.Replace("mb", "");
                panic = panic.ToLower();
                panic = panic.Replace("mb","");
                disk_warn_textbox.Text = warn;
                disk_panic_textbox.Text = panic;

            }
            if (mode == "%")
            {
                disk_panic_textbox.Visible = false;
                disk_panic_trackbar.Visible = true;
                disk_warn_textbox.Visible = false;
                disk_warn_trackbar.Visible = true;
                Disk_MB_warn_RB.Checked = false;
                Disk_percent_warn_rb.Checked = true;
                warn = warn.ToLower();
                panic = panic.ToLower();
                warn = warn.Replace("%", "");
                panic = panic.Replace("%", "");
                disk_warn_trackbar.Value = int.Parse(warn);
                disk_panic_trackbar.Value = int.Parse(panic);
            }
            gb_Disk_settings.Enabled = true;
            disk_remove.Enabled = true;
        }

        private void disk_warn_textbox_TextChanged(object sender, EventArgs e)
        {
            int value = -1;
            try { value = int.Parse(disk_warn_textbox.Text); }
            catch { }
            if (value >= 0)
            {
                build_disk_listview(disk_LV.Items[disk_LV.SelectedIndices[0]].Text, disk_warn_textbox.Text + "mb", disk_panic_textbox.Text + "mb", "change");
            }
        }

        private void disk_panic_textbox_TextChanged(object sender, EventArgs e)
        {
            int value = -1;
            try { value = int.Parse(disk_warn_textbox.Text); }
            catch { }
            if (value >= 0)
            {
                build_disk_listview(disk_LV.Items[disk_LV.SelectedIndices[0]].Text, disk_warn_textbox.Text + "mb", disk_panic_textbox.Text + "mb", "change");
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (disk_LV.Items[disk_LV.SelectedIndices[0]].Text != "default")
            {
                disk_LV.Items.RemoveAt(disk_LV.SelectedIndices[0]);
            }
            else { MessageBox.Show("removal of default prohibited."); }
        }

        private void disk_remove_Click(object sender, EventArgs e)
        {
            if (disk_LV.Items[disk_LV.SelectedIndices[0]].Text != "default")
            {
                remove_xml_node(disk_LV.Items[disk_LV.SelectedIndices[0]].Text);
                disk_LV.Items.RemoveAt(disk_LV.SelectedIndices[0]);
            }
            else { MessageBox.Show("removal of default prohibited."); disk_remove.Enabled = false; }
        }

        private void disk_add_Click(object sender, EventArgs e)
        {
            Add_Disk new_disk = new Add_Disk();
            new_disk.Show();
            string choice = new_disk.list_disks().Replace(":\\","");
            new_disk.Hide();
            build_disk_listview(choice, disk_LV.Items[0].SubItems[1].Text, disk_LV.Items[0].SubItems[2].Text, "add");
            Add_2_part_xml(disk_LV.Items[disk_LV.Items.Count - 1].Text, disk_LV.Items[disk_LV.Items.Count - 1].SubItems[1].Text, disk_LV.Items[disk_LV.Items.Count - 1].SubItems[2].Text, "configuration//disk//setting");
        }
        private void Add_2_part_xml(string text_box_name, string warning_value, string panic_value, string section)
        {
            if (Globals.ready == true)
            {
                Globals.changed = true;
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(Globals.xml_doc);
                XmlNodeList settings = doc.SelectNodes(section);
                XmlNode node = doc.SelectNodes(section.Replace("//setting", ""))[0];
                XmlElement new_element = doc.CreateElement("setting");
                new_element.SetAttribute("name", text_box_name);
                new_element.SetAttribute("warnlevel", warning_value);
                new_element.SetAttribute("paniclevel", panic_value);
                node.AppendChild(new_element);
                StringWriter sw = new StringWriter();
                XmlTextWriter tx = new XmlTextWriter(sw);
                tx.Formatting = Formatting.Indented;
                doc.WriteTo(tx);
                Globals.xml_doc = sw.ToString();
                tx.Flush();
                RTB.Clear();
                RTB.Text = Globals.xml_doc;
            }
        }

        private void Button_Scan_services_On_This_PC_Click(object sender, EventArgs e)
        {
            ServiceController[] services = ServiceController.GetServices();
            foreach (ServiceController service in services)
            {
                add_service(service.DisplayName.ToString());
            }
        }
        private void add_service(string service_name)
        {
            // loop through list of available services looking for this one
            // we only want to add each service 1 time
            bool addit = true;
            for (int i = 0; i < lb_Available_services.Items.Count; i++)
            {
                if (lb_Available_services.Items[i].ToString() == service_name)
                {
                    addit = false;
                }
            }
            if (addit == true) { lb_Available_services.Items.Add(service_name); }
        }

        private void button_add_service_Click(object sender, EventArgs e)
        {
            if (lb_Available_services.SelectedIndex > -1)
            {
                // loop through list of monitored services looking for this one
                // we only want to add a service 1 time
                bool addit = true;
                for (int i = 0; i < LB_Services_to_monitor.Items.Count; i++)
                {
                    if (LB_Services_to_monitor.Items[i].ToString() == lb_Available_services.Items[lb_Available_services.SelectedIndex].ToString())
                    {
                        addit = false;
                    }
                }
                if (addit == true) { LB_Services_to_monitor.Items.Add(lb_Available_services.Items[lb_Available_services.SelectedIndex].ToString()); add_service_to_xml(lb_Available_services.Items[lb_Available_services.SelectedIndex].ToString()); lb_Available_services.Items.RemoveAt(lb_Available_services.SelectedIndex); }
                else { MessageBox.Show("Service : " + lb_Available_services.Items[lb_Available_services.SelectedIndex].ToString() + "\nAlready Exists in the monitored list"); }
            }
        }

        private void button_remove_service_Click(object sender, EventArgs e)
        {
            if (LB_Services_to_monitor.SelectedIndex > -1)
            {
                add_service(LB_Services_to_monitor.Items[LB_Services_to_monitor.SelectedIndex].ToString());
                remove_xml_node(LB_Services_to_monitor.Items[LB_Services_to_monitor.SelectedIndex].ToString());
                LB_Services_to_monitor.Items.RemoveAt(LB_Services_to_monitor.SelectedIndex);
            }
        }

        private void lb_Available_services_SelectedIndexChanged(object sender, EventArgs e)
        {
            label33.Text = "";
            label34.Text = "";
            if (lb_Available_services.SelectedIndex > -1)
            {
                ServiceController[] services = ServiceController.GetServices();
                foreach (ServiceController service in services)
                {
                    if (service.DisplayName.ToString() == lb_Available_services.Items[lb_Available_services.SelectedIndex].ToString())
                    {
                        label33.Text = service.Status.ToString();
                        label34.Text = service.ServiceName.ToString();
                    }
                }
            }
            else {label33.Text = "";label34.Text = "";}
        }
        private void add_service_to_xml(string service_name)
        {
            if (Globals.ready == true)
            {
                Globals.changed = true;
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(Globals.xml_doc);
                XmlNodeList settings = doc.SelectNodes("//configuration//svcs");
                XmlNode node = doc.SelectNodes("//configuration//svcs")[0];
                XmlElement new_element = doc.CreateElement("setting");
                new_element.SetAttribute("name", service_name);
                new_element.SetAttribute("value", "started");
                new_element.SetAttribute("autoreset", "true");
                new_element.SetAttribute("alarmcolor", "red");
                bool doit = true;
                foreach (XmlNode node1 in settings)
                {
                    try
                    {
                        if (node1.Attributes["name"].Value == service_name)
                        {
                            doit = false;
                        }
                    }
                    catch { }
                }
                if (doit == true) { node.AppendChild(new_element); }
                StringWriter sw = new StringWriter();
                XmlTextWriter tx = new XmlTextWriter(sw);
                tx.Formatting = Formatting.Indented;
                doc.WriteTo(tx);
                Globals.xml_doc = sw.ToString();
                tx.Flush();
                RTB.Clear();
                RTB.Text = Globals.xml_doc;
            }
        }

        private void button_add_text_service_Click(object sender, EventArgs e)
        {
            if (tb_add_service.Text != "") { add_service(tb_add_service.Text); tb_add_service.Text = ""; }
        }

        private void button_services_add_all_Click(object sender, EventArgs e)
        {
            bulk_add_services bas = new bulk_add_services();
            bas.Show();
            int choice = bas.what_to_add();
            bas.Hide();
            if (choice == 0)
            {
                for (int i = 0; i < lb_Available_services.Items.Count; i++)
                {
                    bool addit = true;
                    for (int y = 0; y < LB_Services_to_monitor.Items.Count; y++)
                    {
                        if (lb_Available_services.Items[i].ToString() == LB_Services_to_monitor.Items[y].ToString())
                        { addit = false; }
                    }
                    for (int y = 0; y < Globals.floppers.Length; y++)
                    {
                        if (lb_Available_services.Items[i].ToString() == Globals.floppers[y])
                        {
                            DialogResult result = MessageBox.Show("The service " + lb_Available_services.Items[i].ToString() + "\nIs know to cycle between started and stopped.\nAre you sure you would like to add this one ?", "Service known to cycle", MessageBoxButtons.YesNo);
                            if (result == DialogResult.No) { addit = false; }
                        }
                    }
                    if (addit == true)
                    {
                        LB_Services_to_monitor.Items.Add(lb_Available_services.Items[i].ToString());
                        add_service_to_xml(lb_Available_services.Items[i].ToString());
                    }
                }
                lb_Available_services.Items.Clear();
            }
            if (choice == 1)
            {
                bool[] removethese = new bool[lb_Available_services.Items.Count];
                ServiceController[] services = ServiceController.GetServices();
                for (int i = 0; i < lb_Available_services.Items.Count; i++)
                {
                    bool addit = false;
                    foreach (ServiceController service in services)
                    {
                        if (service.DisplayName.ToString() == lb_Available_services.Items[i].ToString())
                        {
                            if (service.Status == ServiceControllerStatus.Running) { addit = true; removethese[i] = true; }
                            else { removethese[i] = false; }
                        }
                    }
                    for (int y = 0; y < LB_Services_to_monitor.Items.Count; y++)
                    {
                        if (lb_Available_services.Items[i].ToString() == LB_Services_to_monitor.Items[y].ToString())
                        { addit = false; }
                    }
                    for (int y = 0; y < Globals.floppers.Length; y++)
                    {
                        if (lb_Available_services.Items[i].ToString() == Globals.floppers[y])
                        {
                            DialogResult result = MessageBox.Show("The service " + lb_Available_services.Items[i].ToString() + "\nIs know to cycle between started and stopped.\nAre you sure you would like to add this one ?", "Service known to cycle", MessageBoxButtons.YesNo);
                            if (result == DialogResult.No) { addit = false; }
                        }
                    }
                    if (addit == true)
                    {
                        LB_Services_to_monitor.Items.Add(lb_Available_services.Items[i].ToString());
                        add_service_to_xml(lb_Available_services.Items[i].ToString());
                    }
                }
                for (int r = lb_Available_services.Items.Count; r > 0; r--)
                {
                    if (removethese[r-1] == true)
                    {
                        lb_Available_services.Items.RemoveAt(r - 1);
                    }
                }
            }
        }

        private void button_download_services_Click(object sender, EventArgs e)
        {
            string services = "";
            using (var client = new WebClient())
            {
                services = client.DownloadString(Globals.services_master_list_url);
            }
            string[] service = services.Split('\n');
            foreach (string svcs in service)
            {
                bool addit = true;
                for (int i = 0; i < lb_Available_services.Items.Count; i++)
                {
                    if (lb_Available_services.Items[i].ToString() == svcs) { addit = false; }
                }
                if (addit == true) { lb_Available_services.Items.Add(svcs); }
            }
        }

        private void Donate_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=WEEYHHMMNK3RN");
        }

        private void gpl3_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.gnu.org/licenses/gpl-3.0-standalone.html");
        }

        private void button2_Click_1(object sender, EventArgs e)
        {
            lb_procs_available.Items.Clear();
            Process[] processlist = Process.GetProcesses();
            for (int i = 0; i < processlist.Length; i++)
            {
                bool addit = true;
                for (int y = 0; y < lb_procs_available.Items.Count; y++)
                {
                    try
                    {
                        string[] proc = processlist[i].MainModule.FileName.ToString().Split('\\');
                        if (lb_procs_available.Items[y].ToString() == proc[proc.Length - 1]) { addit = false; }
                    }
                    catch { }
                }
                if (addit == true) {
                    try
                    {
                        string[] proc = processlist[i].MainModule.FileName.ToString().Split('\\');
                        lb_procs_available.Items.Add(proc[proc.Length - 1]);
                    }
                    catch { }
                }
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            using (var client = new WebClient())
            {
                System.Diagnostics.Process.Start("http://www.processlibrary.com/search/?q=" + lb_procs_available.Items[lb_procs_available.SelectedIndex].ToString());
            }
        }
        private void add_proc(string proc, string rule, string comment)
        {
            string temp_value = proc + "|" + rule + "|" + comment;
            bool addit = true;
            int replace_value = -1;
            for (int i = 0; i < lb_procs_being_monitored.Items.Count; i++)
            {
                if (lb_procs_being_monitored.Items[i].ToString().Contains(proc))
                {
                    addit = false;
                    replace_value = i;
                }
            }
            if (addit == true) { lb_procs_being_monitored.Items.Add(temp_value); }
            if (addit == false) { lb_procs_being_monitored.Items.RemoveAt(replace_value); lb_procs_being_monitored.Items.Add(temp_value); }
            tb_proc_rule.Enabled = true;
            tb_proc_rule.Text = "";
            tb_Proc_description.Text = "";
            proc_rb_min.Checked = false;
            proc_rb_max.Checked = false;
            proc_rb_exact.Checked = false;
            add_proc_to_xml(proc, rule, comment);
        }

        private void button4_Click(object sender, EventArgs e)
        {
            string message = "";
            string comment = tb_Proc_description.Text;
            string rule = "";
            if (lb_procs_available.SelectedIndex < 0) { message = message + "\nSelect an available Process"; }
            if (comment == "") { message = message + "\nProvide a description for process being added"; }
            if (tb_proc_rule.Text == "") { message = message + "\nA rule must be applied, please select number of instances for this proc"; }
            int temp1 = 0;
            try
            {
                temp1 = int.Parse(tb_proc_rule.Text);
            }
            catch
            {
                message = message + "\nA number value must be applied to the process rule";
            }
            if (proc_rb_exact.Checked) { rule = "=" + tb_proc_rule.Text; }
            if (proc_rb_max.Checked) { rule = "-" + tb_proc_rule.Text; }
            if (proc_rb_min.Checked) { rule = "+" + tb_proc_rule.Text; }
            if (message != "")
            {
                MessageBox.Show(message);
                return;
            }
            add_proc(lb_procs_available.Items[lb_procs_available.SelectedIndex].ToString(), rule, comment);
            tb_proc_rule.Enabled = true;
            tb_proc_rule.Text = "";
        }

        private void button3_Click_1(object sender, EventArgs e)
        {
            if (lb_procs_being_monitored.SelectedIndex >= 0)
            {
                string[] temp_value = lb_procs_being_monitored.Items[lb_procs_being_monitored.SelectedIndex].ToString().Split('|');
                bool addit = true;
                for (int i = 0; i < lb_procs_available.Items.Count; i++)
                {
                    if (lb_procs_available.Items[i].ToString() == temp_value[0]) { addit = false; }
                }
                if (addit == true) { lb_procs_available.Items.Add(temp_value[0]); }
                lb_procs_being_monitored.Items.RemoveAt(lb_procs_being_monitored.SelectedIndex);
                remove_xml_node(temp_value[0]);
            }
        }


        private void button5_Click(object sender, EventArgs e)
        {
            if (tb_proc_add_text.Text != "") { lb_procs_available.Items.Add(tb_proc_add_text.Text); tb_proc_add_text.Text = ""; }
        }
        private void add_proc_to_xml(string name,string rule, string comment)
        {
            if (Globals.ready == true)
            {
                Globals.changed = true;
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(Globals.xml_doc);
                XmlNodeList settings = doc.SelectNodes("//configuration//procs");
                XmlNode node = doc.SelectNodes("//configuration//procs")[0];
                XmlElement new_element = doc.CreateElement("setting");
                new_element.SetAttribute("name", name);
                new_element.SetAttribute("rule", rule);
                if (comment != "no description available")
                new_element.SetAttribute("comment", comment);
                new_element.SetAttribute("alarmcolor", "red");
                bool doit = true;
                foreach (XmlNode node1 in settings)
                {
                    try
                    {
                        if (node1.Attributes["name"].Value == name)
                        {
                            doit = false;
                            node1.Attributes["rule"].Value = rule;
                            node1.Attributes["comment"].Value = comment;
                        }
                    }
                    catch { }
                }
                if (doit == true) { node.AppendChild(new_element); }
                StringWriter sw = new StringWriter();
                XmlTextWriter tx = new XmlTextWriter(sw);
                tx.Formatting = Formatting.Indented;
                doc.WriteTo(tx);
                Globals.xml_doc = sw.ToString();
                tx.Flush();
                RTB.Clear();
                RTB.Text = Globals.xml_doc;
            }
        }
        private void remove_xml_node(string name)
        {
            if (Globals.ready == true)
            {
                Globals.changed = true;
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(Globals.xml_doc);
                XmlNodeList nodes = doc.SelectNodes("//setting[@name='"+name+"']");
                for (int i = nodes.Count - 1; i >= 0; i--)
                {
                    nodes[i].ParentNode.RemoveChild(nodes[i]);
                }
                StringWriter sw = new StringWriter();
                XmlTextWriter tx = new XmlTextWriter(sw);
                tx.Formatting = Formatting.Indented;
                doc.WriteTo(tx);
                Globals.xml_doc = sw.ToString();
                tx.Flush();
                RTB.Clear();
                RTB.Text = Globals.xml_doc;
            }
        }

        private void button_xml_save_Click(object sender, EventArgs e)
        {
            string fname = @"c:\";
            if (File.Exists(@"c:\program files\bbwin\etc\bbwin.cfg")) { fname = @"c:\program files\bbwin\etc\"; }
            if (File.Exists(@"c:\program files (x86)\bbwin\etc\bbwin.cfg")) { fname = @"c:\program files (x86)\bbwin\etc\"; }
            SaveFileDialog savedialog = new SaveFileDialog();
            savedialog.Title = "Save As";
            savedialog.DefaultExt = "cfg";
            savedialog.InitialDirectory = fname;
            savedialog.Filter = "cfg files (*.cfg)|*.cfg|bak files (*.bak)|*.bak|All files (*.*)|*.*";
            string filename = "";
            if (savedialog.ShowDialog() == DialogResult.OK)
            {
                filename = savedialog.FileName;
            }
            if (filename != "")
            {
                string backup_filename = backup_config(filename);
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(Globals.xml_doc);
                doc.Save(filename);
                Globals.changed = false;
                if (backup_filename != "")
                { MessageBox.Show("Configuration saved to: " + filename + "\n" + backup_filename); }
                else { MessageBox.Show("Configuration saved to: " + filename); }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            RTB.ReadOnly = false;
            MessageBox.Show("Please ensure any changes you make are valid" + "\nChanges made in this window are not checked for Validity");
        }

        private void proc_rb_max_CheckedChanged(object sender, EventArgs e)
        {
            tb_proc_rule.Text = "1"; tb_proc_rule.Enabled = false;
        }

        private void proc_rb_min_CheckedChanged(object sender, EventArgs e)
        {
            tb_proc_rule.Enabled = true;
        }

        private void proc_rb_exact_CheckedChanged(object sender, EventArgs e)
        {
            tb_proc_rule.Enabled = true;
        }

        private void button_services_clear_Click(object sender, EventArgs e)
        {
            if (LB_Services_to_monitor.Items.Count > 0)
            {
                while (LB_Services_to_monitor.Items.Count > 0)
                {
                    for (int i = 0; i < LB_Services_to_monitor.Items.Count; i++)
                    {
                        add_service(LB_Services_to_monitor.Items[i].ToString());
                        remove_xml_node(LB_Services_to_monitor.Items[i].ToString());
                        LB_Services_to_monitor.Items.RemoveAt(i);
                    }
                }
            }
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            lb_Available_services.Items.Clear();
        }

        private void button6_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.squidworks.net/");
        }
        private string backup_config(string filename)
        {
            string returnvalue = "";
            if (File.Exists(filename))
            {
                string new_file_name = filename + ".BACKUP." + DateTime.Now.ToShortDateString() + "(" + DateTime.Now.ToShortTimeString() + ").bak";
                new_file_name = new_file_name.Replace('/', '-');
                new_file_name = new_file_name.Replace(':', '.');
                new_file_name = new_file_name.Replace("c.\\", "c:\\");
                new_file_name = new_file_name.Replace("C.\\", "C:\\");
                File.Copy(filename, new_file_name);
                returnvalue = "Previous configuration backed up to " + new_file_name;
            }
            return returnvalue;
        }
    }
}
